window.Gleam.initSearch([{"doc":"squirrel","title":"squirrel","content":"# 🐿️ squirrel - type safe SQL in Gleam\n\n[![Package Version](https://img.shields.io/hexpm/v/squirrel)](https://hex.pm/packages/squirrel)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/squirrel/)\n\n[![Squirrel showcase](https://github.com/user-attachments/assets/4abd308b-b191-4296-a0b6-093db0595794)](https://github.com/user-attachments/assets/4abd308b-b191-4296-a0b6-093db0595794)\n\n## What&#39;s Squirrel?\n\nIf you need to talk with a database in Gleam you&#39;ll have to write something like\nthis:\n\n```gleam\nimport pog\nimport decode/zero\n\npub type FindSquirrelRow {\n  FindSquirrelRow(name: String, owned_acorns: Int)\n}\n\n/// Find a squirrel and its owned acorns given its name.\n///\npub fn find_squirrel(db: pog.Connection, name: String) {\n  let squirrel_row_decoder = {\n    use name &lt;- zero.field(0, zero.string)\n    use owned_acorns &lt;- zero.field(1, zero.int)\n    zero.success(FindSquirrelRow(name:, owned_acorns:))\n  }\n\n  &quot;\n  select\n     name,\n     owned_acorns\n   from\n     squirrel\n   where\n     name = $1\n  &quot;\n  |&gt; pog.query\n  |&gt; pog.parameter(pog.text(name))\n  |&gt; pog.returning(zero.run(_, squirrel_row_decoder))\n  |&gt; pog.execute(db)\n}\n```\n\nThis is probably fine if you have a few small queries but it can become quite\nthe burden when you have a lot of queries:\n\n- The SQL query you write is just a plain string, you do not get syntax\n  highlighting, auto formatting, suggestions... all the little niceties you\n  would otherwise get if you where writing a plain `*.sql` file.\n- This also means you loose the ability to run these queries on their own with\n  other external tools, inspect them and so on.\n- You have to manually keep in sync the decoder with the query&#39;s output.\n\nOne might be tempted to hide all of this by reaching for something like an ORM.\nSquirrel proposes a different approach: instead of trying to hide the SQL it\n_embraces it and leaves you in control._\nYou write the SQL queries in plain old `*.sql` files and Squirrel will take care\nof generating all the corresponding functions.\n\nA code snippet is worth a thousand words, so let&#39;s have a look at an example.\nInstead of the hand written example shown earlier you can instead just write the\nfollowing query:\n\n```sql\n-- we&#39;re in file `src/squirrels/sql/find_squirrel.sql`\n-- Find a squirrel and its owned acorns given its name.\nselect\n  name,\n  owned_acorns\nfrom\n  squirrel\nwhere\n  name = $1\n```\n\nAnd run `gleam run -m squirrel`. Just like magic you&#39;ll now have a type-safe\nfunction `find_squirrel` you can use just as you&#39;d expect:\n\n```gleam\nimport pog\nimport squirrels/sql\n\npub fn main() {\n  let db = todo as &quot;the pog connection&quot;\n  // And it just works as you&#39;d expect:\n  let assert Ok(pog.Returned(_rows_count, rows)) = sql.find_squirrel(&quot;sandy&quot;)\n  let assert [FindSquirrelRow(name: &quot;sandy&quot;, owned_acorns: 11_111)] = rows\n}\n```\n\nBehind the scenes Squirrel generates the decoders and functions you need; and\nit&#39;s pretty-printed, standard Gleam code (actually it&#39;s exactly like the hand\nwritten example I showed you earlier)!\nSo now you get the best of both worlds:\n\n- You don&#39;t have to take care of keeping encoders and decoders in sync, Squirrel\n  does that for you.\n- And you&#39;re not compromising on type safety either: Squirrel is able to\n  understand the types of your query and produce a correct decoder.\n- You can stick to writing plain SQL in `*.sql` files. You&#39;ll have better\n  editor support, syntax highlighting and completions.\n- You can run each query on its own: need to `explain` a query?\n  No big deal, it&#39;s just a plain old `*.sql` file.\n\n## Usage\n\nFirst you&#39;ll need to add Squirrel to your project as a dev dependency:\n\n```sh\ngleam add squirrel --dev\n```\n\nThen you can ask it to generate code running the `squirrel` module:\n\n```sh\ngleam run -m squirrel\n```\n\nAnd that&#39;s it! As long as you follow a couple of conventions Squirrel will just\nwork:\n\n- Squirrel will look for all `*.sql` files in any `sql` directory under your\n  project&#39;s `src` directory.\n- Each `sql` directory will be turned into a single Gleam module containing a\n  function for each `*.sql` file inside it. The generated Gleam module is going\n  to be located in the same directory as the corresponding `sql` directory and\n  it&#39;s name is `sql.gleam`.\n- Each `*.sql` file _must contain a single SQL query._ And the name of the file\n  is going to be the name of the corresponding Gleam function to run that query.\n\n&gt; Let&#39;s make an example. Imagine you have a Gleam project that looks like this\n&gt;\n&gt; ```txt\n&gt; ├── src\n&gt; │   ├── squirrels\n&gt; │   │   └── sql\n&gt; │   │       ├── find_squirrel.sql\n&gt; │   │       └── list_squirrels.sql\n&gt; │   └── squirrels.gleam\n&gt; └── test\n&gt;     └── squirrels_test.gleam\n&gt; ```\n&gt;\n&gt; Running `gleam run -m squirrel` will create a `src/squirrels/sql.gleam` file\n&gt; defining two functions `find_squirrel` and `list_squirrels` you can then\n&gt; import and use in your code.\n\n### Talking to the database\n\nIn order to understand the type of your queries, Squirrel needs to connect to\nthe Postgres server where the database is defined. To connect, it will read the\n`DATABASE_URL` env variable that has to be a valid connection string with the\nfollowing format:\n\n```txt\npostgres://user:password@host:port/database\n```\n\nIf a `DATABASE_URL` variable is not set, Squirrel will instead read your\n[Postgres env variables](https://www.postgresql.org/docs/current/libpq-envars.html)\nand use the following defaults if one is not set:\n\n- `PGHOST`: `&quot;localhost&quot;`\n- `PGPORT`: `5432`\n- `PGUSER`: `&quot;root&quot;`\n- `PGDATABASE`: the name of your Gleam project\n- `PGPASSWORD`: `&quot;&quot;`\n\n## Supported types\n\nSquirrel takes care of the mapping between Postgres types and Gleam types.\nThis is needed in two places:\n\n- Gleam values need to be _encoded_ into Postgres values when you&#39;re filling in\n  the holes of a prepared statement (`$1`, `$2`, ...)\n- Postgres values need to be _decoded_ into a Gleam ones when you&#39;re reading the\n  rows returned by a query.\n\nThe types that are currently supported are:\n\n| postgres type                                     | encoded as                                                                                      | decoded as                                                                                      |\n| ------------------------------------------------- | ----------------------------------------------------------------------------------------------- | ----------------------------------------------------------------------------------------------- |\n| `bool`                                            | `Bool`                                                                                          | `Bool`                                                                                          |\n| `text`, `char`, `bpchar`, `varchar`               | `String`                                                                                        | `String`                                                                                        |\n| `float4`, `float8`, `numeric`                     | `Float`                                                                                         | `Float`                                                                                         |\n| `int2`, `int4`, `int8`                            | `Int`                                                                                           | `Int`                                                                                           |\n| `json`, `jsonb`                                   | [`Json`](https://hexdocs.pm/gleam_json/gleam/json.html#Json)                                    | `String`                                                                                        |\n| `uuid`                                            | [`Uuid`](https://hexdocs.pm/youid/youid/uuid.html#Uuid)                                         | [`Uuid`](https://hexdocs.pm/youid/youid/uuid.html#Uuid)                                         |\n| `bytea`                                           | `BitArray`                                                                                      | `BitArray`                                                                                      |\n| `date`                                            | `#(Int, Int, Int)` with `#(year, month, day)`                                                   | `#(Int, Int, Int)` with `#(year, month, day)`                                                   |\n| `timestamp`                                       | `#(#(Int, Int, Int), (#(Int, Int, Int))` with `#(#(year, month, day), #(hour, minute, second))` | `#(#(Int, Int, Int), (#(Int, Int, Int))` with `#(#(year, month, day), #(hour, minute, second))` |\n| `&lt;type&gt;[]` (where `&lt;type&gt;` is any supported type) | `List(&lt;type&gt;)`                                                                                  | `List(&lt;type&gt;)`                                                                                  |\n| user-defined enum                                 | [Gleam custom type](https://tour.gleam.run/data-types/custom-types/)                            | [Gleam custom type](https://tour.gleam.run/data-types/custom-types/)                            |\n\n### Enums\n\nIf your queries deal with user-defined enums Squirrel will automatically turn\neach one of those into a corresponding Gleam type to make sure your code is type\nsafe.\n\nFor example, consider the following enum:\n\n```sql\ncreate type squirrel_colour as enum (\n  &#39;light_brown&#39;,\n  &#39;grey&#39;,\n  &#39;red&#39;\n);\n```\n\nSquirrel turns that into a Gleam type that looks like this:\n\n```gleam\npub type SquirrelColour {\n  LightBrown\n  Grey\n  Red\n}\n```\n\n&gt; Squirrel will convert all the enum name and enum variants into PascalCase to\n&gt; make sure the generated Gleam code is valid.\n&gt; Notice how this transformation might result in having a name that is still not\n&gt; valid Gleam code; for example if you had an enum variant `&#39;1_first&#39;` that\n&gt; would become `1First` which is not valid Gleam!\n&gt;\n&gt; Squirrel won&#39;t try and trim invalid characters from the names and instead will\n&gt; fail letting you know you should change those names into something that can be\n&gt; turned into valid Gleam code.\n\n## FAQ\n\n### What flavour of SQL does squirrel support?\n\nSquirrel only has support for Postgres.\n\n### Why isn&#39;t squirrel configurable in any way?\n\nBy going the &quot;convention over configuration&quot; route, Squirrel enforces that all\nprojects adopting it will always have the same structure.\nIf you need to contribute to a project using Squirrel you&#39;ll immediately know\nwhich directories and modules to look for.\n\nThis makes it easier to get started with a new project and cuts down on all the\nbike shedding: _&quot;Where should I put my queries?&quot;,_\n_&quot;How many queries should go in on file?&quot;,_ ...\n\n## References\n\nThis package draws a lot of inspiration from the amazing\n[yesql](https://github.com/krisajenkins/yesql) and\n[sqlx](https://github.com/launchbadge/sqlx).\n\n## Contributing\n\nIf you think there’s any way to improve this package, or if you spot a bug don’t\nbe afraid to open PRs, issues or requests of any kind! Any contribution is\nwelcome 💜\n","url":"index.html"},{"doc":"squirrel","title":"exit","content":"pub fn exit(n: Int) -&gt; Nil\n\nSynonyms:\nsquirrel.exit\nsquirrel exit","url":"squirrel.html#exit"},{"doc":"squirrel","title":"main","content":"pub fn main() -&gt; Nil\n 🐿️ Performs code generation for your Gleam project.\n\n `squirrel` is not configurable and will discover the queries to generate\n code for by relying on a conventional project&#39;s structure:\n - `squirrel` first looks for all directories called `sql` under the `src`\n   directory of your Gleam project, and reads all the `*.sql` files in there\n   (in glob terms `src/**/sql/*.sql`).\n - Each `*.sql` file _must contain a single query_ as it is turned into a\n   Gleam function with the same name.\n - All functions coming from the same `sql` directory will be grouped under\n   a Gleam file called `sql.gleam` at the same level: given a `src/$PATH/sql`\n   directory, you&#39;ll end up with a generated `src/$PATH/sql.gleam` file.\n\n &gt; ⚠️ In order to generate type safe code, `squirrel` has to connect\n &gt; to your Postgres database. To know what host, user, etc. values to use\n &gt; when connecting, it will read the `DATABASE_URL` env variable that has to\n &gt; be a valid connection string with the following format:\n &gt;\n &gt; txt\n &gt; postgres://user:password@host:port/database\n &gt; \n &gt;\n &gt; If a `DATABASE_URL` variable is not set, Squirrel will instead read your\n &gt; [Postgres env variables](https://www.postgresql.org/docs/current/libpq-envars.html)\n &gt; and use the following defaults if one is not set:\n &gt; - `PGHOST`: `&quot;localhost&quot;`\n &gt; - `PGPORT`: `5432`\n &gt; - `PGUSER`: `&quot;root&quot;`\n &gt; - `PGDATABASE`: the name of your Gleam project\n &gt; - `PGPASSWORD`: `&quot;&quot;`\n\n &gt; ⚠️ The generated code relies on the\n &gt; [`pog`](https://hexdocs.pm/pog/) and\n &gt; [`decode`](https://hexdocs.pm/decode/) packages to work, so make sure to\n &gt; add those as dependencies to your project.\n\nSynonyms:\nsquirrel.main\nsquirrel main","url":"squirrel.html#main"},{"doc":"squirrel","title":"squirrel","content":"","url":"squirrel.html"}]);